@@ -8,18 +8,31 @@ use elliptic_curve::{
     group::{prime::PrimeCurveAffine, GroupEncoding},
     sec1::{self, FromEncodedPoint, ToEncodedPoint},
     subtle::{Choice, ConditionallySelectable, ConstantTimeEq, CtOption},
-    weierstrass::DecompressPoint,
-    AffineArithmetic,
+    zeroize::DefaultIsZeroes,
+    AffineArithmetic, AffineXCoordinate, DecompressPoint, Error, Result,
 };
 
-#[cfg(feature = "zeroize")]
-use elliptic_curve::zeroize::Zeroize;
-
 impl AffineArithmetic for Secp256k1 {
     type AffinePoint = AffinePoint;
 }
 
-/// A point on the secp256k1 curve in affine coordinates.
+#[cfg(feature = "serde")]
+use elliptic_curve::serde::{de, ser, Deserialize, Serialize};
+
+/// secp256k1 curve point expressed in affine coordinates.
+///
+/// # `serde` support
+///
+/// When the `serde` feature of this crate is enabled, the `Serialize` and
+/// `Deserialize` traits are impl'd for this type.
+///
+/// The serialization uses the [SEC1] `Elliptic-Curve-Point-to-Octet-String`
+/// encoding, serialized as binary.
+///
+/// When serialized with a text-based format, the SEC1 representation is
+/// subsequently hex encoded.
+///
+/// [SEC1]: https://www.secg.org/sec1-v2.pdf
 #[derive(Clone, Copy, Debug)]
 #[cfg_attr(docsrs, doc(cfg(feature = "arithmetic")))]
 pub struct AffinePoint {
@@ -74,6 +87,12 @@ impl PrimeCurveAffine for AffinePoint {
     }
 }
 
+impl AffineXCoordinate<Secp256k1> for AffinePoint {
+    fn x(&self) -> FieldBytes {
+        self.x.to_bytes()
+    }
+}
+
 impl ConditionallySelectable for AffinePoint {
     fn conditional_select(a: &AffinePoint, b: &AffinePoint, choice: Choice) -> AffinePoint {
         AffinePoint {
@@ -98,6 +117,8 @@ impl Default for AffinePoint {
     }
 }
 
+impl DefaultIsZeroes for AffinePoint {}
+
 impl PartialEq for AffinePoint {
     fn eq(&self, other: &AffinePoint) -> bool {
         self.ct_eq(other).into()
@@ -106,9 +127,64 @@ impl PartialEq for AffinePoint {
 
 impl Eq for AffinePoint {}
 
-impl AffinePoint {
-    /// Decode this point from a SEC1-encoded point.
-    pub(crate) fn decode(encoded_point: &EncodedPoint) -> CtOption<Self> {
+impl DecompressPoint<Secp256k1> for AffinePoint {
+    fn decompress(x_bytes: &FieldBytes, y_is_odd: Choice) -> CtOption<Self> {
+        FieldElement::from_bytes(x_bytes).and_then(|x| {
+            let alpha = (x * &x * &x) + &CURVE_EQUATION_B;
+            let beta = alpha.sqrt();
+
+            beta.map(|beta| {
+                let y = FieldElement::conditional_select(
+                    &beta.negate(1),
+                    &beta,
+                    beta.normalize().is_odd().ct_eq(&y_is_odd),
+                );
+
+                Self {
+                    x,
+                    y: y.normalize(),
+                    infinity: Choice::from(0),
+                }
+            })
+        })
+    }
+}
+
+impl GroupEncoding for AffinePoint {
+    type Repr = CompressedPoint;
+
+    fn from_bytes(bytes: &Self::Repr) -> CtOption<Self> {
+        EncodedPoint::from_bytes(bytes)
+            .map(|point| CtOption::new(point, Choice::from(1)))
+            .unwrap_or_else(|_| {
+                // SEC1 identity encoding is technically 1-byte 0x00, but the
+                // `GroupEncoding` API requires a fixed-width `Repr`
+                let is_identity = bytes.ct_eq(&Self::Repr::default());
+                CtOption::new(EncodedPoint::identity(), is_identity)
+            })
+            .and_then(|point| Self::from_encoded_point(&point))
+    }
+
+    fn from_bytes_unchecked(bytes: &Self::Repr) -> CtOption<Self> {
+        // No unchecked conversion possible for compressed points
+        Self::from_bytes(bytes)
+    }
+
+    fn to_bytes(&self) -> Self::Repr {
+        let encoded = self.to_encoded_point(true);
+        let mut result = CompressedPoint::default();
+        result[..encoded.len()].copy_from_slice(encoded.as_bytes());
+        result
+    }
+}
+
+impl FromEncodedPoint<Secp256k1> for AffinePoint {
+    /// Attempts to parse the given [`EncodedPoint`] as an SEC1-encoded [`AffinePoint`].
+    ///
+    /// # Returns
+    ///
+    /// `None` value if `encoded_point` is not on the secp256k1 curve.
+    fn from_encoded_point(encoded_point: &EncodedPoint) -> CtOption<Self> {
         match encoded_point.coordinates() {
             sec1::Coordinates::Identity => CtOption::new(Self::identity(), 1.into()),
             sec1::Coordinates::Compact { .. } => {
@@ -140,61 +216,6 @@ impl AffinePoint {
     }
 }
 
-impl DecompressPoint<Secp256k1> for AffinePoint {
-    fn decompress(x_bytes: &FieldBytes, y_is_odd: Choice) -> CtOption<Self> {
-        FieldElement::from_bytes(x_bytes).and_then(|x| {
-            let alpha = (x * &x * &x) + &CURVE_EQUATION_B;
-            let beta = alpha.sqrt();
-
-            beta.map(|beta| {
-                let y = FieldElement::conditional_select(
-                    &beta.negate(1),
-                    &beta,
-                    beta.normalize().is_odd().ct_eq(&y_is_odd),
-                );
-
-                Self {
-                    x,
-                    y: y.normalize(),
-                    infinity: Choice::from(0),
-                }
-            })
-        })
-    }
-}
-
-impl GroupEncoding for AffinePoint {
-    type Repr = CompressedPoint;
-
-    fn from_bytes(bytes: &Self::Repr) -> CtOption<Self> {
-        let tag = bytes[0];
-        let y_is_odd = tag.ct_eq(&sec1::Tag::CompressedOddY.into());
-        let is_compressed_point = y_is_odd | tag.ct_eq(&sec1::Tag::CompressedEvenY.into());
-        Self::decompress(FieldBytes::from_slice(&bytes[1..]), y_is_odd)
-            .and_then(|point| CtOption::new(point, is_compressed_point))
-    }
-
-    fn from_bytes_unchecked(bytes: &Self::Repr) -> CtOption<Self> {
-        // No unchecked conversion possible for compressed points
-        Self::from_bytes(bytes)
-    }
-
-    fn to_bytes(&self) -> Self::Repr {
-        CompressedPoint::clone_from_slice(self.to_encoded_point(true).as_bytes())
-    }
-}
-
-impl FromEncodedPoint<Secp256k1> for AffinePoint {
-    /// Attempts to parse the given [`EncodedPoint`] as an SEC1-encoded [`AffinePoint`].
-    ///
-    /// # Returns
-    ///
-    /// `None` value if `encoded_point` is not on the secp256k1 curve.
-    fn from_encoded_point(encoded_point: &EncodedPoint) -> Option<Self> {
-        Self::decode(encoded_point).into()
-    }
-}
-
 impl ToEncodedPoint<Secp256k1> for AffinePoint {
     fn to_encoded_point(&self, compress: bool) -> EncodedPoint {
         EncodedPoint::conditional_select(
@@ -209,6 +230,22 @@ impl ToEncodedPoint<Secp256k1> for AffinePoint {
     }
 }
 
+impl TryFrom<EncodedPoint> for AffinePoint {
+    type Error = Error;
+
+    fn try_from(point: EncodedPoint) -> Result<AffinePoint> {
+        AffinePoint::try_from(&point)
+    }
+}
+
+impl TryFrom<&EncodedPoint> for AffinePoint {
+    type Error = Error;
+
+    fn try_from(point: &EncodedPoint) -> Result<AffinePoint> {
+        Option::from(AffinePoint::from_encoded_point(point)).ok_or(Error)
+    }
+}
+
 impl From<AffinePoint> for EncodedPoint {
     /// Returns the SEC1 compressed encoding of this point.
     fn from(affine_point: AffinePoint) -> EncodedPoint {
@@ -244,11 +281,27 @@ impl Neg for AffinePoint {
     }
 }
 
-#[cfg(feature = "zeroize")]
-impl Zeroize for AffinePoint {
-    fn zeroize(&mut self) {
-        self.x.zeroize();
-        self.y.zeroize();
+#[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
+impl Serialize for AffinePoint {
+    fn serialize<S>(&self, serializer: S) -> core::result::Result<S::Ok, S::Error>
+    where
+        S: ser::Serializer,
+    {
+        self.to_encoded_point(true).serialize(serializer)
+    }
+}
+
+#[cfg(feature = "serde")]
+#[cfg_attr(docsrs, doc(cfg(feature = "serde")))]
+impl<'de> Deserialize<'de> for AffinePoint {
+    fn deserialize<D>(deserializer: D) -> core::result::Result<Self, D::Error>
+    where
+        D: de::Deserializer<'de>,
+    {
+        EncodedPoint::deserialize(deserializer)?
+            .try_into()
+            .map_err(de::Error::custom)
     }
 }
 
@@ -257,7 +310,7 @@ mod tests {
     use super::AffinePoint;
     use crate::EncodedPoint;
     use elliptic_curve::{
-        group::prime::PrimeCurveAffine,
+        group::{prime::PrimeCurveAffine, GroupEncoding},
         sec1::{FromEncodedPoint, ToEncodedPoint},
     };
     use hex_literal::hex;
@@ -311,16 +364,20 @@ mod tests {
         assert_eq!(res.as_bytes(), UNCOMPRESSED_BASEPOINT);
     }
 
-    #[test]
-    fn decompress() {
-        let encoded = EncodedPoint::from_bytes(COMPRESSED_BASEPOINT).unwrap();
-        let decompressed = encoded.decompress().unwrap();
-        assert_eq!(decompressed.as_bytes(), UNCOMPRESSED_BASEPOINT);
-    }
-
     #[test]
     fn affine_negation() {
         let basepoint = AffinePoint::generator();
         assert_eq!((-(-basepoint)), basepoint);
     }
+
+    #[test]
+    fn identity_encoding() {
+        // This is technically an invalid SEC1 encoding, but is preferable to panicking.
+        assert_eq!([0; 33], AffinePoint::identity().to_bytes().as_slice());
+        assert!(bool::from(
+            AffinePoint::from_bytes(&AffinePoint::identity().to_bytes())
+                .unwrap()
+                .is_identity()
+        ))
+    }
 }
