@@ -22,7 +22,7 @@
 //!
 //! // Signing
 //! let signing_key = SigningKey::random(&mut OsRng); // Serialize with `::to_bytes()`
-//! let verify_key = signing_key.verify_key();
+//! let verify_key = signing_key.verifying_key();
 //! let message = b"ECDSA proves knowledge of a secret number in the context of a single message";
 //!
 //! // Note: the signature type must be annotated or otherwise inferrable as
@@ -35,11 +35,9 @@
 //! # }
 //! ```
 
-use core::{
-    convert::{TryFrom, TryInto},
-    fmt::{self, Debug},
-};
-use ecdsa_core::{signature::Signature as _, Error};
+use core::fmt::{self, Debug};
+use ecdsa_core::{signature::Signature as _, Error, Result};
+use elliptic_curve::subtle::Choice;
 
 #[cfg(feature = "ecdsa")]
 use crate::{
@@ -48,10 +46,12 @@ use crate::{
         VerifyingKey,
     },
     elliptic_curve::{
-        consts::U32, generic_array::GenericArray, ops::Invert, subtle::Choice,
-        weierstrass::DecompressPoint,
+        bigint::U256,
+        consts::U32,
+        ops::{Invert, LinearCombination, Reduce},
+        DecompressPoint,
     },
-    lincomb, AffinePoint, FieldBytes, NonZeroScalar, ProjectivePoint, Scalar,
+    AffinePoint, FieldBytes, NonZeroScalar, ProjectivePoint, Scalar,
 };
 
 #[cfg(feature = "keccak256")]
@@ -80,10 +80,7 @@ impl Signature {
     ///
     /// This is an "unchecked" conversion and assumes the provided [`Id`]
     /// is valid for this signature.
-    pub fn new(signature: &super::Signature, recovery_id: Id) -> Result<Self, Error> {
-        #[cfg(feature = "ecdsa")]
-        super::check_scalars(signature)?;
-
+    pub fn new(signature: &super::Signature, recovery_id: Id) -> Result<Self> {
         let mut bytes = [0u8; SIZE];
         bytes[..64].copy_from_slice(signature.as_ref());
         bytes[64] = recovery_id.0;
@@ -109,7 +106,7 @@ impl Signature {
         public_key: &VerifyingKey,
         msg: &[u8],
         signature: &super::Signature,
-    ) -> Result<Self, Error> {
+    ) -> Result<Self> {
         Self::from_digest_trial_recovery(public_key, Keccak256::new().chain(msg), signature)
     }
 
@@ -122,12 +119,11 @@ impl Signature {
         public_key: &VerifyingKey,
         digest: D,
         signature: &super::Signature,
-    ) -> Result<Self, Error>
+    ) -> Result<Self>
     where
         D: Clone + Digest<OutputSize = U32>,
     {
-        let mut signature = *signature;
-        signature.normalize_s()?;
+        let signature = signature.normalize_s().unwrap_or(*signature);
 
         for recovery_id in 0..=1 {
             if let Ok(recoverable_signature) = Signature::new(&signature, Id(recovery_id)) {
@@ -151,7 +147,7 @@ impl Signature {
     #[cfg(all(feature = "ecdsa", feature = "keccak256"))]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
     #[cfg_attr(docsrs, doc(cfg(feature = "keccak256")))]
-    pub fn recover_verify_key(&self, msg: &[u8]) -> Result<VerifyingKey, Error> {
+    pub fn recover_verify_key(&self, msg: &[u8]) -> Result<VerifyingKey> {
         self.recover_verify_key_from_digest(Keccak256::new().chain(msg))
     }
 
@@ -159,7 +155,7 @@ impl Signature {
     /// [`VerifyingKey`] from the provided precomputed [`Digest`].
     #[cfg(feature = "ecdsa")]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
-    pub fn recover_verify_key_from_digest<D>(&self, msg_digest: D) -> Result<VerifyingKey, Error>
+    pub fn recover_verify_key_from_digest<D>(&self, msg_digest: D) -> Result<VerifyingKey>
     where
         D: Digest<OutputSize = U32>,
     {
@@ -174,45 +170,45 @@ impl Signature {
     pub fn recover_verify_key_from_digest_bytes(
         &self,
         digest_bytes: &FieldBytes,
-    ) -> Result<VerifyingKey, Error> {
+    ) -> Result<VerifyingKey> {
         let r = self.r();
         let s = self.s();
-        let z = Scalar::from_bytes_reduced(digest_bytes);
+        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(*digest_bytes);
         let R = AffinePoint::decompress(&r.to_bytes(), self.recovery_id().is_y_odd());
 
-        if R.is_some().into() {
-            let R = ProjectivePoint::from(R.unwrap());
-            let r_inv = r.invert().unwrap();
-            let u1 = -(r_inv * z);
-            let u2 = r_inv * *s;
-            let pk = lincomb(&ProjectivePoint::generator(), &u1, &R, &u2).to_affine();
-
-            // TODO(tarcieri): ensure the signature verifies?
-            Ok(VerifyingKey::from(&pk))
-        } else {
-            Err(Error::new())
+        if R.is_none().into() {
+            return Err(Error::new());
         }
+
+        let R = ProjectivePoint::from(R.unwrap());
+        let r_inv = r.invert().unwrap();
+        let u1 = -(r_inv * z);
+        let u2 = r_inv * *s;
+        let pk = ProjectivePoint::lincomb(&ProjectivePoint::generator(), &u1, &R, &u2).to_affine();
+
+        // TODO(tarcieri): ensure the signature verifies?
+        Ok(VerifyingKey::from(&pk))
     }
 
     /// Parse the `r` component of this signature to a [`NonZeroScalar`]
     #[cfg(feature = "ecdsa")]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
     pub fn r(&self) -> NonZeroScalar {
-        NonZeroScalar::from_repr(GenericArray::clone_from_slice(&self.bytes[..32]))
-            .unwrap_or_else(|| unreachable!("r-component ensured valid in constructor"))
+        NonZeroScalar::try_from(&self.bytes[..32])
+            .expect("r-component ensured valid in constructor")
     }
 
     /// Parse the `s` component of this signature to a [`NonZeroScalar`]
     #[cfg(feature = "ecdsa")]
     #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
     pub fn s(&self) -> NonZeroScalar {
-        NonZeroScalar::from_repr(GenericArray::clone_from_slice(&self.bytes[32..64]))
-            .unwrap_or_else(|| unreachable!("s-component ensured valid in constructor"))
+        NonZeroScalar::try_from(&self.bytes[32..64])
+            .expect("s-component ensured valid in constructor")
     }
 }
 
 impl ecdsa_core::signature::Signature for Signature {
-    fn from_bytes(bytes: &[u8]) -> Result<Self, Error> {
+    fn from_bytes(bytes: &[u8]) -> Result<Self> {
         bytes.try_into()
     }
 }
@@ -242,7 +238,7 @@ impl PartialEq for Signature {
 impl TryFrom<&[u8]> for Signature {
     type Error = Error;
 
-    fn try_from(bytes: &[u8]) -> Result<Self, Error> {
+    fn try_from(bytes: &[u8]) -> Result<Self> {
         if bytes.len() != SIZE {
             return Err(Error::new());
         }
@@ -280,7 +276,7 @@ pub struct Id(pub(super) u8);
 
 impl Id {
     /// Create a new [`Id`] from the given byte value
-    pub fn new(byte: u8) -> Result<Self, Error> {
+    pub fn new(byte: u8) -> Result<Self> {
         match byte {
             0 | 1 => Ok(Self(byte)),
             _ => Err(Error::new()),
@@ -288,7 +284,6 @@ impl Id {
     }
 
     /// Is `y` odd?
-    #[cfg(feature = "ecdsa")]
     fn is_y_odd(self) -> Choice {
         self.0.into()
     }
@@ -297,7 +292,7 @@ impl Id {
 impl TryFrom<u8> for Id {
     type Error = Error;
 
-    fn try_from(byte: u8) -> Result<Self, Error> {
+    fn try_from(byte: u8) -> Result<Self> {
         Self::new(byte)
     }
 }
@@ -308,11 +303,30 @@ impl From<Id> for u8 {
     }
 }
 
+impl TryFrom<ecdsa_core::RecoveryId> for Id {
+    type Error = Error;
+
+    fn try_from(id: ecdsa_core::RecoveryId) -> Result<Id> {
+        if id.is_x_reduced() {
+            Err(Error::new())
+        } else if id.is_y_odd() {
+            Ok(Id(1))
+        } else {
+            Ok(Id(0))
+        }
+    }
+}
+
+impl From<Id> for ecdsa_core::RecoveryId {
+    fn from(id: Id) -> ecdsa_core::RecoveryId {
+        ecdsa_core::RecoveryId::new(id.is_y_odd().into(), false)
+    }
+}
+
 #[cfg(all(test, feature = "ecdsa", feature = "sha256"))]
 mod tests {
     use super::Signature;
     use crate::EncodedPoint;
-    use core::convert::TryFrom;
     use hex_literal::hex;
     use sha2::{Digest, Sha256};
 
