@@ -1,25 +1,44 @@
-//! ECDSA verifier
+//! ECDSA verification support.
 
 use super::{recoverable, Error, Signature};
 use crate::{
-    lincomb, AffinePoint, CompressedPoint, EncodedPoint, ProjectivePoint, PublicKey, Scalar,
-    Secp256k1,
+    AffinePoint, CompressedPoint, EncodedPoint, ProjectivePoint, PublicKey, Scalar, Secp256k1,
 };
-use core::convert::TryFrom;
 use ecdsa_core::{hazmat::VerifyPrimitive, signature};
-use elliptic_curve::{consts::U32, ops::Invert, sec1::ToEncodedPoint};
+use elliptic_curve::{
+    bigint::U256,
+    consts::U32,
+    ops::{Invert, LinearCombination, Reduce},
+    sec1::ToEncodedPoint,
+    IsHigh,
+};
 use signature::{digest::Digest, DigestVerifier};
 
 #[cfg(feature = "sha256")]
 use signature::PrehashSignature;
 
 #[cfg(feature = "pkcs8")]
-use crate::pkcs8::{self, FromPublicKey};
+use crate::pkcs8::{self, DecodePublicKey};
 
 #[cfg(feature = "pem")]
 use core::str::FromStr;
 
+#[cfg(all(feature = "pem", feature = "serde"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "pem", feature = "serde"))))]
+use elliptic_curve::serde::{de, ser, Deserialize, Serialize};
+
 /// ECDSA/secp256k1 verification key (i.e. public key)
+///
+/// # `serde` support
+///
+/// When the `serde` feature of this crate is enabled, the `Serialize` and
+/// `Deserialize` traits are impl'd for this type.
+///
+/// The serialization is binary-oriented and supports ASN.1 DER-encoded
+/// X.509 Subject Public Key Info (SPKI) as the encoding format.
+///
+/// For a more text-friendly encoding of public keys, use
+/// [`elliptic_curve::JwkEcKey`] instead.
 #[cfg_attr(docsrs, doc(cfg(feature = "ecdsa")))]
 #[derive(Copy, Clone, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub struct VerifyingKey {
@@ -78,7 +97,7 @@ where
 }
 
 impl VerifyPrimitive<Secp256k1> for AffinePoint {
-    fn verify_prehashed(&self, z: &Scalar, signature: &Signature) -> Result<(), Error> {
+    fn verify_prehashed(&self, z: Scalar, signature: &Signature) -> Result<(), Error> {
         let r = signature.r();
         let s = signature.s();
 
@@ -88,10 +107,10 @@ impl VerifyPrimitive<Secp256k1> for AffinePoint {
         }
 
         let s_inv = s.invert().unwrap();
-        let u1 = z * &s_inv;
+        let u1 = z * s_inv;
         let u2 = *r * s_inv;
 
-        let x = lincomb(
+        let x = ProjectivePoint::lincomb(
             &ProjectivePoint::generator(),
             &u1,
             &ProjectivePoint::from(*self),
@@ -100,7 +119,7 @@ impl VerifyPrimitive<Secp256k1> for AffinePoint {
         .to_affine()
         .x;
 
-        if Scalar::from_bytes_reduced(&x.to_bytes()).eq(&r) {
+        if <Scalar as Reduce<U256>>::from_be_bytes_reduced(x.to_bytes()).eq(&r) {
             Ok(())
         } else {
             Err(Error::new())
@@ -118,7 +137,7 @@ impl From<PublicKey> for VerifyingKey {
 
 impl From<&PublicKey> for VerifyingKey {
     fn from(public_key: &PublicKey) -> VerifyingKey {
-        public_key.clone().into()
+        VerifyingKey::from(*public_key)
     }
 }
 
@@ -130,7 +149,7 @@ impl From<VerifyingKey> for PublicKey {
 
 impl From<&VerifyingKey> for PublicKey {
     fn from(verifying_key: &VerifyingKey) -> PublicKey {
-        verifying_key.inner.clone().into()
+        verifying_key.inner.into()
     }
 }
 
@@ -140,9 +159,17 @@ impl From<&AffinePoint> for VerifyingKey {
     }
 }
 
+impl From<ecdsa_core::VerifyingKey<Secp256k1>> for VerifyingKey {
+    fn from(verifying_key: ecdsa_core::VerifyingKey<Secp256k1>) -> VerifyingKey {
+        VerifyingKey {
+            inner: verifying_key,
+        }
+    }
+}
+
 impl From<&VerifyingKey> for EncodedPoint {
-    fn from(verify_key: &VerifyingKey) -> EncodedPoint {
-        verify_key.to_encoded_point(true)
+    fn from(verifying_key: &VerifyingKey) -> EncodedPoint {
+        verifying_key.to_encoded_point(true)
     }
 }
 
@@ -162,12 +189,18 @@ impl TryFrom<&EncodedPoint> for VerifyingKey {
 
 #[cfg(feature = "pkcs8")]
 #[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
-impl FromPublicKey for VerifyingKey {
-    fn from_spki(spki: pkcs8::SubjectPublicKeyInfo<'_>) -> pkcs8::Result<Self> {
-        PublicKey::from_spki(spki).map(|pk| Self { inner: pk.into() })
+impl TryFrom<pkcs8::SubjectPublicKeyInfo<'_>> for VerifyingKey {
+    type Error = pkcs8::spki::Error;
+
+    fn try_from(spki: pkcs8::SubjectPublicKeyInfo<'_>) -> pkcs8::spki::Result<Self> {
+        PublicKey::try_from(spki).map(|pk| Self { inner: pk.into() })
     }
 }
 
+#[cfg(feature = "pkcs8")]
+#[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
+impl DecodePublicKey for VerifyingKey {}
+
 #[cfg(feature = "pem")]
 #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
 impl FromStr for VerifyingKey {
@@ -178,6 +211,28 @@ impl FromStr for VerifyingKey {
     }
 }
 
+#[cfg(all(feature = "pem", feature = "serde"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "pem", feature = "serde"))))]
+impl Serialize for VerifyingKey {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: ser::Serializer,
+    {
+        self.inner.serialize(serializer)
+    }
+}
+
+#[cfg(all(feature = "pem", feature = "serde"))]
+#[cfg_attr(docsrs, doc(cfg(all(feature = "pem", feature = "serde"))))]
+impl<'de> Deserialize<'de> for VerifyingKey {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: de::Deserializer<'de>,
+    {
+        ecdsa_core::VerifyingKey::<Secp256k1>::deserialize(deserializer).map(Into::into)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::VerifyingKey;
@@ -194,8 +249,8 @@ mod tests {
         let verifying_key = VerifyingKey::from_sec1_bytes(&verifying_key_bytes).unwrap();
 
         let msg = hex!("313233343030");
-        let mut sig = Signature::from_der(&hex!("304402207fffffffffffffffffffffffffffffff5d576e7357a4501ddfe92f46681b20a002207fffffffffffffffffffffffffffffff5d576e7357a4501ddfe92f46681b20a0")).unwrap();
-        assert!(!sig.normalize_s().unwrap());
+        let sig = Signature::from_der(&hex!("304402207fffffffffffffffffffffffffffffff5d576e7357a4501ddfe92f46681b20a002207fffffffffffffffffffffffffffffff5d576e7357a4501ddfe92f46681b20a0")).unwrap();
+        assert!(sig.normalize_s().is_none()); // Ensure signature is already normalized
         assert!(verifying_key.verify(&msg, &sig).is_ok());
     }
 }
